#! /usr/bin/perl -w

#==========================================================================
# Summary
#==========================================================================
# Hardware test on HP Proliant platform
# 
# This script check hardware on HP Proliant platform. You must install
# HP insight agent on each supervised server.
# This script as been tested with:
# HP DL380 series under Windows
# HP DL380 series under Linux
# 
#
# Copyright (C) 2006 APHP 
# Copyright (C) 2006 LINAGORA <http://www.linagora.com>
# 
#==========================================================================
# License: GPLv2
#==========================================================================
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# GPL License: http://www.gnu.org/licenses/gpl.txt
#
#==========================================================================
# Please visit http://www.linagora.org for other contributions
#==========================================================================

#==========================================================================
# Changelog
#==========================================================================
# Version 0.1 (2007/11/20):
# - First version
# Author: APHP
#==========================================================================

#==========================================================================
# Version
#==========================================================================
my $VERSION = '0.1';
my $TEMPLATE_VERSION = '1.0.0';

use strict;
use Getopt::Long;
use lib '/usr/lib/nagios/plugins/hp';
use utils qw /$TIMEOUT %ERRORS &print_revision &support/;
&Getopt::Long::config('bundling');
use File::Basename;
use POSIX;
use Net::SNMP;

######## CONFIGURATION (ADD NEW OIDs HERE)
my (%SNMP_OID)=(
    'PWR' => ".1.3.6.1.4.1.232.6.2.9.1",
    'THR' => ".1.3.6.1.4.1.232.6.2.6.1",
    'MEM' => ".1.3.6.1.4.1.232.6.2.14.4",
    'CPU' => ".1.3.6.1.4.1.232.1.2.2.1.1.6",
    'PDRV' => ".1.3.6.1.4.1.232.3.2.5.1.1.37",
    'LDRV' => "1.3.6.1.4.1.232.3.2.3.1.1.11" ,
    'SFAN' => ".1.3.6.1.4.1.232.6.2.6.4",
    'CFAN' => ".1.3.6.1.4.1.232.6.2.6.5"
    );

my (%DESCRIPTORS)=(
    'PWR' => "Check power supply",
    'THR' => "Check system temperature",
    'MEM' => "Check system memory",
    'CPU' => "Check processors",
    'PDRV' => "Check physical drive",
    'LDRV' => "Check logical drive" ,
    'SFAN' => "Check system fan",
    'CFAN' => "Check cpu fan"
    );

######## END OF CONFIGURATION

#######
#
# Can manage 5 states: Other, Ok, Degraded, Failed, Disabled
#
########

my %check_states   = (1 => "Other",2 => "OK",3 => "Degraded",4 => "Failed",5 => "Disabled",);
my %states = ('Other' => 0,'Ok' => 0,'Degraded' => 0,'Failed' => 0, 'Disabled' => 0 );

#==========================================================================
# Options
#==========================================================================
my $progname = basename($0);
my $help;
my $version;
my $verbose = 0;
my $host;
my $warning;
my $critical;
my $logname;
my $authentication;
my $log_file;
my $perf_data;
my $name;
my $port;
my $timeout = $TIMEOUT;
my $regexp;
my $eregexp;
my $exclude;
my $minute = 60;
my $url;
# For SNMP plugins
my $snmp_version="2c";
my $snmp_v1;
my $snmp_v2c;
my $snmp_v3;
my $community="public";
my $seclevel;
my $authproto;
my $privpasswd;
my $type;
my $list_type;

GetOptions(
	   'h'=> \$help,'help'=> \$help,
	   'V'=> \$version,'version'=> \$version,
	   'v+'=> \$verbose,'verbose+'=> \$verbose,
	   'H:s'=> \$host,'host:s'=> \$host,
#	   'w:f'=> \$warning,'warning:f'=> \$warning,
#	   'c:f'=> \$critical,'critical:f'    => \$critical,
#	   'l:s'=> \$logname,'logname:s'=> \$logname,
#	   'a:s'=> \$authentication,
#	   'authentication:s'=> \$authentication,
#	   'F:s'=> \$log_file,'log_file:s'=> \$log_file,
#	   'f'=> \$perf_data,'perf_data'=> \$perf_data,
#	   'n:s'=> \$name,'name:s'=> \$name,
#	   'p:i'=> \$port,'port:i'=> \$port,
#	   't:i'=> \$timeout,'timeout:i'=> \$timeout,
#	   'r:s'=> \$regexp,'regexp:s'=> \$regexp,
#	   'e:s'=> \$exclude,'exclude:s'=> \$exclude,
#	   'm:s'=> \$minute,'minute:s'=> \$minute,
#	   'u:s'=> \$url,'url:s'=> \$url,
	   'l'=> \$list_type,'list_type'=> \$list_type,
	   # For SNMP plugins
	   'snmp_version:s'=> \$snmp_version,'1'=> \$snmp_v1,
	   '2'=> \$snmp_v2c,'3'=> \$snmp_v3,
	   'C:s'=> \$community,'community:s'=> \$community,
	   'L:s'=> \$seclevel,'seclevel:s'=> \$seclevel,
	   'A:s'=> \$authproto,'authproto:s'=> \$authproto,
	   'X:s'=> \$privpasswd,'privpasswd:s'=> \$privpasswd,
	   'T:s'=> \$type,'type:s'=> \$type
	   );

# Fix SMNP Version
unless ($snmp_version) {
    $snmp_version = "1" if $snmp_v1;
    $snmp_version = "2c" if $snmp_v2c;
    $snmp_version = "3" if $snmp_v3;
}

#==========================================================================
# Usage
#==========================================================================
sub print_usage {
    print "Usage: \n";
    print "$progname -H <hostname> -T <check_type> [-h] [-v] [-V]\n\n";
    print "Use option --help for more information\n\n";
    print "$progname comes with ABSOLUTELY NO WARRANTY\n\n";
}

#==========================================================================
# List type of plugins
#==========================================================================
sub print_plugins_list {
    &print_revision($progname, "\$Revision: $VERSION (TPL: $TEMPLATE_VERSION)\$");

    print "\n\nPlugins list:\n";
    foreach (keys %DESCRIPTORS)
    {
	print "$_ ==> $DESCRIPTORS{$_}\n";
    }

    exit $ERRORS{'UNKNOWN'};
}

#=========================================================================
# Version
#=========================================================================
if ($version) {
    &print_revision($progname, "\$Revision: $VERSION (TPL: $TEMPLATE_VERSION)\$");
    exit $ERRORS{'UNKNOWN'};
}

#=========================================================================
# Help
#=========================================================================
if ($help) {
    &print_revision($progname, "\$Revision: $VERSION\$");
    
    &print_usage;
    
    print " This script check hardware on HP Proliant platform. You must install\n";
    print " HP insight agent on each supervised server.\n";
    print " This script as been tested with:\n";
    print " HP DL380 series under Windows\n";
    print " HP DL380 series under Linux\n\n";

    print "-v, --verbose\n";
    print "\tPrint extra debugging information.\n";
    print "-V, --version\n";
    print "\tPrint version and exit.\n";
    print "-h, --help\n";
    print "\tPrint this help message and exit.\n";
    print "-H, --host=STRING\n";
    print "\tIP or name (FQDN).\n";
#    print "-p, --port=INTEGER\n";
#    print "\tPort to connect to.\n";
#    print "-w, --warning=DOUBLE\n";
#    print "\t Level to return a warning status.\n";
#    print "-c, --critical=DOUBLE\n";
#    print "\tLevel to return a critical status.\n";
#    print "-l, --logname=STRING\n";
#    print "\tUser id for login.\n";
#    print "-a, --authentication=STRING\n";
#    print "\tSSH private key file.\n";
#    print "-F, --log_file=STRING\n";
#    print "\tStatus or log file.\n";
#    print "-f, --perf_data\n";
#    print "\tDisplay performance data.\n";
#    print "-n, --name=STRING\n";
#    print "\tName (database, table, service, ...).\n";
#    print "-t, --timeout=INTEGER\n";
#    print "\tSeconds before connection times out (default: $TIMEOUT).\n";
#    print "-r, --regexp=STRING\n";
#    print "\tCase sensitive regular expression.\n";
#    print "-e, --exclude=STRING\n";
#    print "\nExclude this regular expression.\n";
#    print "-u, --url=STRING\n";
#    print "\tURL.\n";
    print "-1, -2  --snmp_version=(1|2c)\n";
    print "\tSNMP protocol version.\n";
    print "-C, --community=STRING\n";
    print "\tSNMP community (v1 and v2c).\n";
#    print "-L, --seclevel=(noAuthNoPriv|authNoPriv|authPriv))\n";
#    print "\tSNMP security level (v3).\n";
#    print "-A, --authproto=(MD5|SHA)\n";
#    print "\tSNMP authentication protocol (v3).\n";
#    print "-X, --privpasswd=STRING\n";
#    print "\tSNMP cipher password (v3).\n";
    print "-o, --oid=STRING\n";
    print "\tSNMP OID.\n";
    print "-l, --list_type\n";
    print "\tList plugins available.\n";
    print "-T, --type=STRING\n";
    print "\tSelect plugin.\n";
    print "\n";

    &support;

    exit $ERRORS{'UNKNOWN'};
}

if ($list_type)
{
    &print_plugins_list();
    exit $ERRORS{'UNKNOWN'};
}

#=========================================================================
# Functions
#=========================================================================

# DEBUG function
sub verbose {
    my $output_code = shift ;
    my $text = shift ;
    if ( $verbose >= $output_code ) {
        printf "VERBOSE $output_code ===> %s\n", $text;
    }
} 


# check if -H is used 
sub check_host_param{
    if ( !defined( $host) ) {
        printf "UNKNOWN: you have to define a hostname.\n";
        exit $ERRORS{UNKNOWN};
    }

    if ( !defined( $type) ) {
        printf "UNKNOWN: you have to define a plugin to use.\n";
        exit $ERRORS{UNKNOWN};
    }
}

# check_state.
sub check_state {
    my $state='UNKNOWN';
    my ($snmp_session, $SNMP_ERROR) = Net::SNMP->session(
						 -hostname  => $host,
						 -community => $community,
						 -version => $snmp_version,
                                                 -port => "161"
						 );
    if (!defined($snmp_session))
    {
	printf("ERROR on host: %s.\n", $SNMP_ERROR);
	exit $ERRORS{"UNKNOWN"};
    }
    my ($table) = $snmp_session->get_table(-baseoid => $SNMP_OID{$type});

    if (!defined($table))
    {
	printf("ERROR on host: %s can't get SNMP table\n", $host);
	exit $ERRORS{"UNKNOWN"};
    }

    my ($oid);
    my ($nb)=0;
    foreach $oid (keys %{$table})
    {
	$states{$check_states{$table->{$oid}}}++;
	$nb++;
    }
    
    print "$type ";
    foreach $_ (keys %states)
    {
	if ( $states{$_} )
	{
	if ($_ =~ /Failed/i)
	{
	    $state = 'CRITICAL';
	}
	if (($_ =~ /Degraded/i) &&( $state !~ /CRITICAL/i))
	{
	    $state = 'WARNING';
	}
	if (($_ =~ /Ok/i) && ($state !~ /CRITICAL/i) && ($state !~ /WARNING/i))
	{
	    $state = 'OK';
	}
	print "$_ $states{$_}/$nb ";
	}
    }
    print "\n";

    $snmp_session->close;
    return $state;
}

#=========================================================================
# Main
#=========================================================================


&check_host_param();

my $check_state = &check_state();

#==========================================================================
# Exit with Nagios codes
#==========================================================================

#
# Utiliser la variable $ERRORS pour les codes de retour
# Tester l'option PerfParse (-f)
# Indiquer les valeurs seuil dans le message (-w et -c)
# 

exit $ERRORS{$check_state};
